# AMPL-SML code for solving linear or quadratic separable minimum cost 
# multicommodity flow problems, using BlockIP

param K >0; #number of commodities
set COMMODITIES := 1..K;
param M >0; #number of nodes
set NODES := 1..M;
set ARCS within {NODES,NODES};

param injection {COMMODITIES,NODES} ; #supply or demand at nodes
            	                    #supply >0  demand<0

param cost{COMMODITIES,ARCS} ; #linear costs
param qcost{COMMODITIES,ARCS} >= 0; #quadratic convex diagonal costs
param ind_cap{COMMODITIES,ARCS}; #individual capacities per commodity
param mut_cap{ARCS}; # mutual capacity at arcs

block Comm{k in COMMODITIES}:
  var flow{(i,j) in ARCS} >=0, <= ind_cap[k,i,j];
 
  minimize Total_Cost: sum {(i,j) in ARCS} 
	  (cost[k,i,j] + qcost[k,i,j]*flow[i,j])*flow[i,j];

  subject to Balance {m in NODES}:
	  sum{(i,j) in ARCS : i==m} flow[i,j]
	 -sum{(i,j) in ARCS : j==m} flow[i,j] = injection[k,m];
end block;

subject to MutCap {(i,j) in ARCS} :
	sum {k in COMMODITIES} Comm[k].flow[i,j] <= mut_cap[i,j];

